@@ -25,6 +25,7 @@ use bellframe::{
 };
 use log::{log_enabled, LevelFilter};
 use monument::{Comp, Config, Progress, Query, QueryUpdate};
+use ordered_float::OrderedFloat;
 use ringing_utils::{BigNumInt, PrettyDuration};
 use simple_logger::SimpleLogger;
 use spec::Spec;
@@ -40,7 +41,7 @@ pub fn init_logging(filter: LevelFilter) {
 
 pub fn run(
     input_file: &Path,
-    debug_print: Option<DebugOption>,
+    debug_option: Option<DebugOption>,
     queue_limit: usize,
 ) -> Result<Option<QueryResult>, Error> {
     let start_time = Instant::now();
@@ -49,7 +50,7 @@ pub fn run(
     /// corresponding value and exit.
     macro_rules! debug_print {
         ($variant: ident, $val: expr) => {
-            if debug_print == Some(DebugOption::$variant) {
+            if debug_option == Some(DebugOption::$variant) {
                 dbg!($val);
                 return Ok(None);
             }
@@ -78,7 +79,7 @@ pub fn run(
     let graph = query.unoptimised_graph();
     debug_print!(Graph, graph);
     let optimised_graphs = query.optimise_graph(graph, &mut config);
-    if debug_print == Some(DebugOption::StopBeforeSearch) {
+    if debug_option == Some(DebugOption::StopBeforeSearch) {
         return Ok(None);
     }
 
@@ -100,7 +101,7 @@ pub fn run(
     );
 
     let mut comps = comps.lock().unwrap().to_vec();
-    comps.sort_by_key(|comp| comp.avg_score);
+    comps.sort_by_key(|comp| OrderedFloat(comp.music_score()));
     Ok(Some(QueryResult {
         comps,
         duration: Instant::now() - start_time,
@@ -117,7 +118,7 @@ impl QueryResult {
     pub fn print(&self) {
         println!("\n\n\n\nSEARCH COMPLETE!\n\n\n");
         for c in &self.comps {
-            println!("{}", c.long_string());
+            println!("{}", c);
         }
         println!("Search completed in {}", PrettyDuration(self.duration));
     }
@@ -130,6 +131,7 @@ pub enum Error {
     MusicFile(PathBuf, spec::TomlReadError),
 
     PartHeadParse(InvalidRowError),
+    // TODO: For too short inputs, suggest expanding with either bells or a `*`
     ChMaskParse(String, bellframe::mask::ParseError),
     ChPatternParse(String, bellframe::mask::ParseError),
 
@@ -213,7 +215,7 @@ impl SingleLineProgressLogger {
         // generated).
         let mut update_string = String::new();
         if let Some(c) = &comp {
-            update_string.push_str(&c.long_string());
+            update_string.push_str(&c.to_string());
             update_string.push('\n');
         }
         self.append_progress_string(&mut update_string);
