@@ -1,12 +1,14 @@
 use std::{
     collections::HashMap,
-    ops::Range,
+    ops::{Deref, Range},
     path::{Path, PathBuf},
 };
 
 use bellframe::{
-    method::LABEL_LEAD_END, method_lib::QueryError, music::Regex, Bell, Mask, MethodLib, Row,
-    RowBuf, Stage, Stroke,
+    method::LABEL_LEAD_END,
+    method_lib::QueryError,
+    music::{Regex, RegexElem},
+    Bell, Mask, MethodLib, Row, RowBuf, Stage, Stroke,
 };
 use itertools::Itertools;
 use monument::{
@@ -40,7 +42,7 @@ pub struct Spec {
     /// The range of lengths of composition which are allowed
     length: Length,
     /// Monument won't stop until it generates the `num_comps` best compositions
-    #[serde(default = "get_30")]
+    #[serde(default = "default_num_comps")]
     num_comps: usize,
     /// Allow Monument to ignore falseness and generate false compositions.  Compositions still
     /// won't have internal rounds.
@@ -97,9 +99,12 @@ pub struct Spec {
     split_tenors: bool,
     /// Which course heads masks are allowed (overrides `split_tenors`)
     course_heads: Option<Vec<String>>,
-    /// Weights applied to given CH patterns
+    /// Score applied to every row with a given CH patterns
     #[serde(default)]
     ch_weights: Vec<ChWeightPattern>,
+    /// Weight given to every row in a course, for every handbell pair that's coursing
+    #[serde(default)]
+    handbell_coursing_weight: f32,
     /// If `true`, generate compositions lead-wise, rather than course-wise.  This is useful for
     /// cases like cyclic comps where no course heads are preserved across parts.
     ///
@@ -112,12 +117,14 @@ pub struct Spec {
     #[serde(default)]
     snap_start: bool,
     /// Which indices within a lead should the composition be allowed to start.  If unspecified,
-    /// then all locations are allowed
+    /// then all locations are allowed.  All indices are taken modulo each method's lead length (so
+    /// 2, -30 and 34 are all equivalent for Treble Dodging Major).
     #[serde(default = "default_start_indices")]
-    start_indices: Option<Vec<usize>>,
+    start_indices: Option<Vec<isize>>,
     /// Which indices within a lead should the composition be allowed to start.  If unspecified,
-    /// then all locations are allowed
-    end_indices: Option<Vec<usize>>,
+    /// then all locations are allowed.  All indices are taken modulo each method's lead length (so
+    /// 2, -30 and 34 are all equivalent for Treble Dodging Major).
+    end_indices: Option<Vec<isize>>,
 }
 
 impl Spec {
@@ -148,6 +155,9 @@ impl Spec {
         // `layout::new::Method`s
         let ch_masks = self.ch_mask_preset(stage)?.into_masks(stage);
         let calls = self.calls(stage)?;
+        let part_head =
+            RowBuf::parse_with_stage(&self.part_head, stage).map_err(Error::PartHeadParse)?;
+        let (start_indices, end_indices) = self.start_end_indices(&part_head);
         let shorthands = monument::utils::default_shorthands(
             loaded_methods
                 .iter()
@@ -159,11 +169,17 @@ impl Spec {
                 .course_heads
                 .map(|chs| parse_ch_masks(&chs, stage))
                 .transpose()?;
+            #[allow(clippy::or_fun_call)] // `start_indices.as_deref()` is a really cheap operation
+            let start_indices = common.start_indices.as_deref().or(start_indices.as_deref());
+            #[allow(clippy::or_fun_call)]
+            let end_indices = common.end_indices.as_deref().or(end_indices.as_deref());
             methods.push(layout::new::Method::new(
                 method,
                 calls.clone(),
                 override_ch_masks.unwrap_or_else(|| ch_masks.clone()),
                 shorthand,
+                start_indices,
+                end_indices,
             ));
         }
 
@@ -172,21 +188,11 @@ impl Spec {
         if methods.len() > 1 {
             log::info!("Method count range: {:?}", method_count_range);
         }
-        let ch_weights = self.ch_weights(stage, methods.iter().map(|m| m.method()))?;
+        let ch_weights = self.ch_weights(stage, methods.iter().map(Deref::deref))?;
 
         // Generate the `Layout`
-        let part_head =
-            RowBuf::parse_with_stage(&self.part_head, stage).map_err(Error::PartHeadParse)?;
-        let (start_indices, end_indices) = self.start_end_indices(&part_head);
-        let layout = Layout::new(
-            methods,
-            self.splice_style,
-            &part_head,
-            self.leadwise,
-            start_indices.as_deref(),
-            end_indices.as_deref(),
-        )
-        .map_err(Error::LayoutGen)?;
+        let layout = Layout::new(methods, self.splice_style, &part_head, self.leadwise)
+            .map_err(Error::LayoutGen)?;
 
         let music_types = self.music_types(toml_path, stage)?;
         if music_types.is_empty() {
@@ -261,7 +267,7 @@ impl Spec {
         })
     }
 
-    fn start_end_indices(&self, part_head: &Row) -> (Option<Vec<usize>>, Option<Vec<usize>>) {
+    fn start_end_indices(&self, part_head: &Row) -> (Option<Vec<isize>>, Option<Vec<isize>>) {
         let mut start_indices = self.start_indices.clone().or_else(|| {
             if self.snap_start {
                 None // If just `snap_start`, then allow any start
@@ -318,8 +324,15 @@ impl Spec {
             .flat_map(|m| m.lead_head().closure())
             .unique()
             .collect_vec();
-
         let mut weights = Vec::new();
+        // Closure to a weight pattern for every CH which contains a lead which matches `mask`
+        let mut add_ch_pattern = |mask: &Mask, weight: f32| {
+            for lh in &lead_heads {
+                weights.push((mask * lh, weight));
+            }
+        };
+
+        // Add explicit patterns from the `ch_weights` parameter
         for pattern in &self.ch_weights {
             // Extract a (slice of patterns, weight)
             use ChWeightPattern::*;
@@ -327,12 +340,27 @@ impl Spec {
                 Pattern { pattern, weight } => (std::slice::from_ref(pattern), weight),
                 Patterns { patterns, weight } => (patterns.as_slice(), weight),
             };
+            // Add the patterns
             for mask_str in ch_masks {
                 let mask = Mask::parse_with_stage(mask_str, stage)
                     .map_err(|e| Error::ChPatternParse(mask_str.to_owned(), e))?;
-                // Add a weight pattern for every CH which contains a lead which matches `mask`
-                for lh in &lead_heads {
-                    weights.push((&mask * lh, *weight));
+                add_ch_pattern(&mask, *weight);
+            }
+        }
+        // Add patterns from `handbell_coursing_weight`
+        if self.handbell_coursing_weight != 0.0 {
+            for right_bell in stage.bells().step_by(2) {
+                let left_bell = right_bell + 1;
+                // For every handbell pair, we need patterns for `*<left><right>` and `*<right><left>`
+                for (b1, b2) in [(left_bell, right_bell), (right_bell, left_bell)] {
+                    let regex = Regex::from_elems([
+                        RegexElem::Glob,
+                        RegexElem::Bell(b1),
+                        RegexElem::Bell(b2),
+                    ]);
+                    let mask = Mask::from_regex(&regex, stage)
+                        .expect("Handbell patterns should always be valid");
+                    add_ch_pattern(&mask, self.handbell_coursing_weight);
                 }
             }
         }
@@ -409,7 +437,7 @@ pub enum MethodSpec {
     },
 }
 
-/// Common values for methods
+/// Common values for all method variants
 #[derive(Debug, Clone, Default, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct MethodCommon {
@@ -419,6 +447,8 @@ pub struct MethodCommon {
     lead_locations: Option<HashMap<String, String>>,
     /// Custom set of course head masks for this method
     course_heads: Option<Vec<String>>,
+    start_indices: Option<Vec<isize>>,
+    end_indices: Option<Vec<isize>>,
 }
 
 impl MethodSpec {
@@ -508,7 +538,7 @@ pub struct MusicFile {
 pub enum MusicSpec {
     RunLength {
         #[serde(rename = "run_length")]
-        length: usize,
+        length: u8,
         #[serde(default)]
         internal: bool,
         #[serde(flatten)]
@@ -516,7 +546,7 @@ pub enum MusicSpec {
     },
     RunLengths {
         #[serde(rename = "run_lengths")]
-        lengths: Vec<usize>,
+        lengths: Vec<u8>,
         #[serde(default)]
         internal: bool,
         #[serde(flatten)]
@@ -560,7 +590,7 @@ impl MusicSpec {
     pub fn to_music_types(&self, stage: Stage, default_non_duffer: bool) -> Vec<MusicType> {
         enum LoweredType<'_self> {
             /// Equivalent to [`Self::Runs`] or [`Self::RunsList`]
-            Runs(&'_self [usize], &'_self bool),
+            Runs(&'_self [u8], &'_self bool),
             /// Equivalent to [`Self::Pattern`] or [`Self::Pattern`]
             Patterns(&'_self [String], &'_self OptRange),
         }
@@ -650,8 +680,8 @@ fn get_one() -> f32 {
     1.0
 }
 
-fn get_30() -> usize {
-    30
+fn default_num_comps() -> usize {
+    100
 }
 
 fn get_true() -> bool {
@@ -670,7 +700,7 @@ fn default_lead_labels() -> HashMap<String, String> {
     labels
 }
 
-fn default_start_indices() -> Option<Vec<usize>> {
+fn default_start_indices() -> Option<Vec<isize>> {
     Some(vec![0]) // Just normal starts
 }
 
